package jesse.keeblarcraft.gui.ShopKeeperGUI;

import jesse.keeblarcraft.Entities.ShopKeeper;
import jesse.keeblarcraft.GuiMgr.ShopKeeper.ShopKeeperHandler;
import jesse.keeblarcraft.GuiMgr.ShopKeeper.ShopKeeperInventory;
import jesse.keeblarcraft.Keeblarcraft;
import jesse.keeblarcraft.ClientHelpers.Helper;
import jesse.keeblarcraft.gui.Generics.ClickableLayer;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.gui.screen.ingame.HandledScreen;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.text.Text;
import net.minecraft.util.Identifier;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ShopKeeperMenu extends HandledScreen<ShopKeeperHandler> {
    private static final Identifier MENU_TEXTURE = new Identifier(Keeblarcraft.MOD_ID, "textures/gui/shopkeeper_gui.png");
    private static final Identifier ADD_BUTTON = new Identifier(Keeblarcraft.MOD_ID, "textures/gui/plus.png");
    private static final Identifier SCROLLBAR = new Identifier(Keeblarcraft.MOD_ID, "textures/gui/scrollbar.png");
    private int x = 0;
    private int y = 0;
    private final List<ClickableLayer> layers = new ArrayList<>();
    private MerchandiseScroller catalogue;
    private ShopKeeperInventory serverInventory;
    Scrollbar scrollbar;

    public ShopKeeperMenu(ShopKeeperHandler handler, PlayerInventory inventory, Text title) {
        super(handler, inventory, title);
    }

    @Override
    protected void init() {
        super.init();
        titleY = 1000; //begone from screen
        playerInventoryTitleY = 1000; //begone from screen

        // Set the main shop keeping menu variables here
        this.backgroundWidth = 256;
        this.backgroundHeight = 256;
        this.x = (width - backgroundWidth) / 2;
        this.y = (height - backgroundHeight) / 2;
        scrollbar = new Scrollbar(SCROLLBAR, Text.of("scrollbar"), 15, 28, x+243, y+40, y+50, 163);

        // Add all layers below
        AddNewButton(ADD_BUTTON, Text.of("ADD ITEM BUTTON"), x+20, y+20, 8, 8);
        AddNewLayer(scrollbar);

        catalogue = new MerchandiseScroller(this.getScreenHandler().GetInventory());
        System.out.println("This store catalogue has size " + catalogue.Size());
    }

    @Override
    protected void drawBackground(DrawContext context, float delta, int mouseX, int mouseY) {
        // Draw the menu texture first
//        context.drawTexture(MENU_TEXTURE, this.x, this.y, 0, 0, this.backgroundWidth, this.backgroundHeight);
        context.drawTexture(MENU_TEXTURE, this.x, this.y, 0, 0, this.backgroundWidth, this.backgroundHeight, this.backgroundWidth, this.backgroundHeight);
//        context.drawTexture(NEW_ITEM_MENU, this.x, this.y, 0, 0, 32, 24);
//        context.drawTexture(NEW_ITEM_MENU, this.x, this.y, 0, 0, 32, 24, 32, 24);

        this.getScreenHandler().showSlot("add_item_menu", "add_item_slot");

        //  TODO: Temporary to just prove a point that we can draw item icons in the menu
        ItemStack tempStack = Items.ACACIA_DOOR.getDefaultStack();


        // Draws primary layers first
        for (ClickableLayer layer : layers) {
            layer.renderButton(context, mouseX, mouseY, delta);
            ToggleSlots(layer.GetLayerName().getString(), layer.GetDrawableSlots());

            // If a primary layer happens to want sub-layers in itself; we draw those here
            for (ClickableLayer subLayer : layer.GetSubLayers()) {
                subLayer.renderButton(context, mouseX, mouseY, delta);
                ToggleSlots(subLayer.GetLayerName().getString(), subLayer.GetDrawableSlots());
            }

            // TODO: HACK to see if this works
            context.drawItem(tempStack, this.x, this.y); // TODO: REMOVE
            // We only notify the parent to update sub layers. It must manage its own sub layers
            layer.UpdateSubLayers();
        }
    }

    private void ToggleSlots(String menuName, Map<String, Boolean> slotNames) {
        for (Map.Entry<String, Boolean> slot : slotNames.entrySet()) {
            if (slot.getValue()) {
//                System.out.println("Attempting to turn slot " + slot.getKey() + " to value " + slot.getValue());
                this.getScreenHandler().showSlot(menuName, slot.getKey());
            } else {
//                System.out.println("Attempting to turn slot " + slot.getKey() + " to value " + slot.getValue());
                this.getScreenHandler().hideSlot(menuName, slot.getKey());
            }
        }
    }

    // Takes an image and adds it to the background draw list and saves coordinates to return to a callback if it is clicked
    private void AddNewButton(Identifier iconId, Text layerName, int drawX, int drawY, int imgWidth, int imgHeight) {
        layers.add(new NewItemButton(iconId, layerName, imgWidth, imgHeight, drawX, drawY));
    }

    private void AddNewLayer(ClickableLayer layer) {
        layers.add(layer);
    }

    @Override
     public void drawForeground(DrawContext context, int mouseX, int mouseY) {
         // We override this function to intentionally do nothing
         // If in the future we want, we would draw the foreground and TITLES with this!
     }

    @Override
    public boolean mouseClicked(double mouseX, double mouseY, int button) {
        for (ClickableLayer layer : layers) {
            // Check main layer first
            if (Helper.WithinBounds(layer.getX(), layer.getY(), layer.getWidth(), layer.getHeight(), mouseX, mouseY)) {
                layer.mouseClicked(mouseX, mouseY, button);
            }

            for (ClickableLayer subLayer : layer.GetSubLayers()) {

                if (subLayer.visible && Helper.WithinBounds(subLayer.getX(), subLayer.getY(), subLayer.getWidth(), subLayer.getHeight(), mouseX, mouseY)) {
                    subLayer.mouseClicked(mouseX, mouseY, button);
                }
            }
        }

        return true;
    }

    @Override
    public boolean mouseReleased(double mouseX, double mouseY, int button) {
        scrollbar.SetDragging(false);

        for (ClickableLayer layer : layers) {
            // Scrollbar gets special handling above this. Ignore in loop
            if (layer.GetLayerName().getString().equals("scrollbar")) {
                continue;
            }
            // Check main layer first
            if (Helper.WithinBounds(layer.getX(), layer.getY(), layer.getWidth(), layer.getHeight(), mouseX, mouseY)) {
                layer.mouseReleased(mouseX, mouseY, button);
            }

            for (ClickableLayer subLayer : layer.GetSubLayers()) {

                if (subLayer.visible && Helper.WithinBounds(subLayer.getX(), subLayer.getY(), subLayer.getWidth(), subLayer.getHeight(), mouseX, mouseY)) {
                    subLayer.mouseReleased(mouseX, mouseY, button);
                }
            }
        }

        return true;
    }

    @Override
    public boolean mouseScrolled(double mouseX, double mouseY, double horizontalAmount, double verticalAmount) {
        System.out.println("Mouse scrolled. Vert amount is: " + verticalAmount);
        scrollbar.mouseScrolled(mouseX, mouseY, horizontalAmount, verticalAmount);
        return true;
    }



    @Override
    public boolean mouseDragged(double mouseX, double mouseY, int button, double deltaX, double deltaY) {
//        super.mouseDragged(mouseX, mouseY, button, deltaX, deltaY);
        if (scrollbar.IsDragging()) {
            scrollbar.mouseDragged(mouseX, mouseY, button, deltaX, deltaY);
        }

        for (ClickableLayer layer : layers) {
            // Scrollbar layer gets special handling above this for loop. Skip here
            if (layer.GetLayerName().getString().equals("scrollbar")) {
                continue;
            }

            // Check main layer first
            if (Helper.WithinBounds(layer.getX(), layer.getY(), layer.getWidth(), layer.getHeight(), mouseX, mouseY)) {
                layer.mouseDragged(mouseX, mouseY, button, deltaX, deltaY);
            }

            for (ClickableLayer subLayer : layer.GetSubLayers()) {

                if (subLayer.visible && Helper.WithinBounds(subLayer.getX(), subLayer.getY(), subLayer.getWidth(), subLayer.getHeight(), mouseX, mouseY)) {
                    subLayer.mouseDragged(mouseX, mouseY, button, deltaX, deltaY);
                }
            }
        }

        return true;
    }

    // Check if a click point is within the context of a square (square = x, y, w, h)
//    public static boolean WithinBounds(double x, double y, double w, double h, double clickX, double clickY) {
////        System.out.println("Mouse click X Y: " + clickX + " " + clickY);
////        System.out.println("Rectangle X Y W H: " + x + " " + y + " " + w + " " + h);
//        return (clickX >= x) && (clickX <= x + w) && (clickY >= y) && (clickY <= y + h);
//    }

    @Override
    public void render(DrawContext context, int mouseX, int mouseY, float delta) {
        renderBackground(context, mouseX, mouseY, delta);
        super.render(context, mouseX, mouseY, delta);
        drawMouseoverTooltip(context, mouseX, mouseY);
    }
}
